import aiohttp
import asyncio
import json
import os

DATA_URL = "https://gitlab.com/Dimbreath/AnimeGameData/-/raw/main/{FOLDER}/{value}"
LANG_URL = "https://gitlab.com/Dimbreath/AnimeGameData/-/raw/main/{LANG_FOLDER}/TextMap{key}.json"
_PATH = None

from .pathfinding import search

from .config import (
    FOLDER,
    LANG_FOLDER,
    AVATAR,
    COSTUME,
    SKILLDEPOT,
    SKILLS,
    TALENTS,
    ARTIFACTS,
    WEAPONS,
    ARTIFACTS_SETS,
    FIGHT_PROPS,
    PROPS_MAP,
    NAMECARDS,
    ARTIFACT_PROPS_MAIN,
    ARTIFACT_PROPS_SUB,
    ENVKEY,
    LANGKEY
)

_DATA = {}
_DATA_LANG = {
    "artifact_sets": [],
    "artifacts": [],
    "characters": [],
    "constellations": [],
    "fight_props": [],
    "namecards": [],
    "skills": [],
    "weapons": [],
}

def save_json(NAME_JSON, data, path = "data"):
    global _PATH
        
    with open(f"{_PATH}/{path}/{NAME_JSON}", "w", encoding="utf-8") as file:
        json.dump(data, file, indent=4, ensure_ascii=False)

async def fetch_data(url, key, lang = False):
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            if response.status == 200:
                data = await response.text()
                _DATA[key] = json.loads(data)
    if lang:
        print(f"| Dowload lang: {key}")

async def creat_artifact_sets():
    data = {}
    for key in _DATA["ARTIFACTS_SETS"]:
        data[key["id"]] = {"affixId": key["affixId"], "nameTextMapHash": key["nameTextMapHash"]}
        _DATA_LANG["artifact_sets"].append(str(key["nameTextMapHash"]))
        
async def creat_weapon():   
    data = {}
    
    for key in _DATA["WEAPONS"]:
        data[key["id"]] = {"nameTextMapHash": key["nameTextMapHash"], "icon": key["icon"], "awakenIcon": key["awakenIcon"], "rankLevel": key["rankLevel"]}
        _DATA_LANG["weapons"].append(str(key["nameTextMapHash"]))
    save_json("weapons.json", data)

async def creat_namecards():   
    data = {}
       
    for key in _DATA["NAMECARDS"]:
        if key.get("materialType", "_") == "MATERIAL_NAMECARD":
            data[key["id"]] = {"nameTextMapHash": key["nameTextMapHash"], "icon": key["icon"], "picPath": key["picPath"], "rankLevel": key["rankLevel"], "materialType": key["materialType"]}
            _DATA_LANG["namecards"].append(str(key["nameTextMapHash"]))
    save_json("namecards.json", data)

async def creat_artifact():
    data = {}
    
    for key in _DATA["ARTIFACTS"]:
        data[key["id"]] = {"nameTextMapHash": key["nameTextMapHash"], "itemType": key["itemType"], "equipType": key["equipType"], "icon": key["icon"], "rankLevel": key["rankLevel"], "mainPropDepotId": key["mainPropDepotId"], "appendPropDepotId": key["appendPropDepotId"]}
        
        _DATA_LANG["artifacts"].append(str(key["nameTextMapHash"]))
        
    save_json("artifacts.json", data)
    
async def creat_costume():   
    data = {}

    for key in _DATA["COSTUME"]:
        data[key["skinId"]] = {"iconName": key["frontIconName"], "sideIconName": key["sideIconName"], "nameTextMapHash": key["nameTextMapHash"]}
    
    save_json("costumes.json", data)
        
async def creat_fight_props():
    data = {}
    
    for key in _DATA["FIGHT_PROPS"]:
        data[key["textMapId"]] = {"nameTextMapHash": key["textMapContentTextMapHash"]}
    save_json("fight_props.json", data)
        
async def creat_skill():
    data = {}
    
    for key in _DATA["SKILLS"]:
        if key["skillIcon"]:
            data[key["id"]] = {"nameTextMapHash": key["nameTextMapHash"], "skillIcon": key["skillIcon"], "proudSkillGroupId": key.get("proudSkillGroupId","")}
            _DATA_LANG["skills"].append(str(key["nameTextMapHash"]))
    save_json("skills.json", data)

async def creat_const():
    data = {}
    
    for key in _DATA["TALENTS"]:
        data[key["talentId"]] = {"nameTextMapHash": key["nameTextMapHash"], "icon": key["icon"]}
        _DATA_LANG["constellations"].append(str(key["nameTextMapHash"]))
    save_json("constellations.json", data)
    
async def creat_skill_depo():
    
    data = {}
    
    for key in _DATA["SKILLDEPOT"]:
        if key["id"] == 101 or key.get("energySkill",0) != 0:
            data[key["id"]] = {"id":key["id"],"energySkill": key["energySkill"], "skills": key["skills"], "subSkills": key["subSkills"], "talents": key["talents"]}
            
    return data

async def creat_characrer():
 
    data = {}
    sub_data = await creat_skill_depo()
   
    skill_data = {}
    for key in _DATA["SKILLS"]:
        skill_data[key["id"]] = {"nameTextMapHash": key["nameTextMapHash"], "skillIcon": key["skillIcon"], "proudSkillGroupId": key.get("proudSkillGroupId",""), "costElemType": key.get("costElemType", None), "forceCanDoSkill": key.get("forceCanDoSkill", None)}
   
    for key in _DATA["AVATAR"]:
        if key["skillDepotId"] == 101 or \
            key["iconName"].endswith("_Kate") or \
            str(key['id'])[:2] == "11":
                continue    
        
        _DATA_LANG["characters"].append(str(key["nameTextMapHash"]))
           
        if key["iconName"].endswith("_PlayerBoy") or key["iconName"].endswith("_PlayerGirl"):
            new_data =  {"nameTextMapHash": key["nameTextMapHash"], "iconName": key["iconName"], "sideIconName": key["sideIconName"], "qualityType": key["qualityType"], 
                           "costElemType": "", "skills": [], "talents": [] }
            
            for skill_id in key["candSkillDepotIds"]:
                new_data.update({"skills": [],"talents": []})
                
                depot = sub_data.get(skill_id)
                if depot and depot["id"] != 101:
                    for skill in depot["skills"]:
                        if skill <= 0:
                            continue
                        new_data["skills"].append(skill)
                        
                    element_id = skill_data.get(depot.get("energySkill"))
                    
                    new_data.update({
                            "costElemType": element_id.get("costElemType")
                        })
                    
                    new_data["skills"].append(int(depot.get('energySkill')))   

                    new_data.update({
                        "talents": [x for x in depot["talents"] if x > 0],
                    })
                    
                    data[str(key["id"]) + "-" + str(depot["id"])] = new_data.copy()
  
            data[str(key["id"])] = new_data.copy()
        else:
            data[key["id"]] = {"nameTextMapHash": key["nameTextMapHash"], "iconName": key["iconName"], "sideIconName": key["sideIconName"], "qualityType": key["qualityType"], 
                           "costElemType": "", "skills": [], "talents": [] }
            depot = sub_data.get(key["skillDepotId"])
            if depot and depot["id"] != 101:
                for skill in depot["skills"]:
                    if skill <= 0:
                        continue
                    skill_info = skill_data.get(int(skill))
                    if not skill_info["forceCanDoSkill"] is None:
                        continue
                    data[key["id"]]["skills"].append(skill)
                element_id = skill_data.get(depot.get("energySkill"))
                data[key["id"]]["costElemType"] = element_id.get("costElemType")
                
                data[key["id"]]["skills"].append(int(depot.get('energySkill')))  
                data[key["id"]]["talents"] =  [x for x in depot["talents"] if x > 0]
                
    save_json("characters.json", data)

async def creat_artifact_prop():
    data = {}
    ARTIFACT_PROPS = []
    ARTIFACT_PROPS.extend(_DATA["ARTIFACT_PROPS_MAIN"])
    ARTIFACT_PROPS.extend(_DATA["ARTIFACT_PROPS_SUB"])
    
    PERCENT = ['HURT','CRITICAL','EFFICIENCY','PERCENT','ADD']
    
        
    for key in ARTIFACT_PROPS:
        ISPERCENT = key['propType'].split("_")[-1] in PERCENT
        RAW = key.get("propValue", 0)

        data[key["id"]] = {
            'propType': key['propType'],
            'propDigit': 'PERCENT' if ISPERCENT else 'DIGIT',
            'propValue': round(RAW * 100, 1) if ISPERCENT else round(RAW)
        }
    
    save_json("artifact_props.json", data)

async def creat_lang():
    task = []
    for key in LANGKEY:
        url = LANG_URL.format(LANG_FOLDER = LANG_FOLDER, key = key)
        task.append(fetch_data(url, key, lang= True))
    print(">> Start dowload lang")
    await asyncio.gather(*task)
    
    print("| End dowload lang")
    
    print(">> Start update lang")
    
    data = {
        "artifact_sets": {},
        "artifacts": {},
        "characters": {},
        "constellations": {},
        "namecards": {},
        "skills": {},
        "weapons": {}
    }
    
    set_lang = {"CHS": "",
        "CHT": "",
        "DE": "",
        "EN": "",
        "ES": "",
        "FR": "",
        "ID": "",
        "IT": "",
        "JP": "",
        "KR": "",
        "PT": "",
        "RU": "",
        "TH": "",
        "TR": "",
        "VI": ""
    }
    
    count = 1
    
    for name in data:
        for key in _DATA_LANG[name]:
            data[name][key] = {
                "CHS": _DATA["CHS"].get(key,""),
                "DE": _DATA["DE"].get(key,""),
                "EN": _DATA["EN"].get(key,""),
                "ES": _DATA["ES"].get(key,""),
                "FR": _DATA["FR"].get(key,""),
                "ID": _DATA["ID"].get(key,""),
                "IT": _DATA["IT"].get(key,""),
                "JP": _DATA["JP"].get(key,""),
                "KR": _DATA["KR"].get(key,""),
                "PT": _DATA["PT"].get(key,""),
                "RU": _DATA["RU"].get(key,""),
                "TH": _DATA["TH"].get(key,""),
                "TR": _DATA["TR"].get(key,""),
                "VI": _DATA["VI"].get(key,""),
            }
        print(f"| ({count}/7) Finish Convert {name}") 
        count += 1

    for key in data:
        save_json(f"{key}.json", data[key], path= "langs")
        
    print("| End update lang")
    
async def dowload(path = None):
    global _PATH
    
    if path is None:
        _PATH = await search()
    
    print(f"Path to enkanetwork.py: {_PATH}")
    print("="*25)
    print(">> Start dowload data")
    variables = {key: globals()[key] for key in ENVKEY}
    task = []
    for key, value in variables.items():
        url = DATA_URL.format(FOLDER = FOLDER, value = value)
        task.append(fetch_data(url, key))
        
    await asyncio.gather(*task)
    print("| End dowload data")
    
    print(">> Start update data")
    
    await asyncio.gather(creat_artifact_sets(),
                        creat_weapon(),
                        creat_namecards(),
                        creat_artifact(),
                        creat_costume(),
                        creat_fight_props(),
                        creat_skill(),
                        creat_const(),
                        creat_characrer(),
                        creat_artifact_prop(),
    )
    
    print("| End update data")
    
    print("="*25)
    
    await creat_lang()
    
    print("="*25)



from pydantic import BaseModel,Field
from typing import List, Optional,Dict,Union
from PIL import Image


class CharacterRank(BaseModel):
    id: Optional[int]
    rank: Optional[int]
    out: Optional[int]
    precent: Optional[int]

class AkashaRank(BaseModel):
    akasha: list[CharacterRank]


class Info(BaseModel):
    uid: Optional[str]
    lang: Optional[str]
    save: Optional[bool]

class Card(BaseModel):
    id: Optional[int]
    name: Optional[str]
    element: Optional[str]
    rarity: Optional[int]
    card: Optional[Union[str, bytes, Image.Image]]

    class Config:
        arbitrary_types_allowed = True

class PickleSize(BaseModel):
    name: Optional[str]
    size: Optional[int]
    text: Optional[str]

class EnkaCard(BaseModel):
    info: Optional[Info]
    card: Optional[List[Card]]
    character_id: Optional[List[str]]
    character_name: Optional[List[str]]
    pickle_size: Optional[List[PickleSize]]
    
    
    async def get_charter(self, setting = False, name = False):
        if setting:
            card_ids = [str(card.id) for card in self.card]

            if name:
                return {name: id for id, name in zip(self.character_id, self.character_name) if id in card_ids}
            return {id: name for id, name in zip(self.character_id, self.character_name) if id in card_ids}
        
        if name:
            return {name: id for id, name in zip(self.character_id, self.character_name)}
        return {id: name for id, name in zip(self.character_id, self.character_name)}

from typing import List, Optional
from pydantic import BaseModel, HttpUrl

class Player(BaseModel):
    name: Optional[str]
    uid: Optional[str]
    lang: Optional[str]
    achievement: Optional[int]
    level: Optional[int]
    world_level: Optional[int]
    abyss: Optional[str]
    avatar: Optional[HttpUrl]

class Characters(BaseModel):
    count: Optional[int]
    character_name: Optional[List[str]]
    character_id: Optional[List[str]]

    async def get_charter(self,name = False):
        if name:
            return {name: id for id, name in zip(self.character_id, self.character_name)}
        return {id: name for id, name in zip(self.character_id, self.character_name)}
    
class Profile(BaseModel):
    player: Optional[Player]
    characters: Optional[Characters]
    card: Optional[Union[str, bytes, Image.Image]]

    class Config:
        arbitrary_types_allowed = True



import aiohttp
from ..utils import pill, git
from PIL import ImageDraw,Image

_of = git.ImageCache()

api_url = "https://akasha.cv/api/getCalculationsForUser/{uid}"
api_update = "https://akasha.cv/api/user/refresh/{uid}"

data_akasha = {}

class AkashaCreat:
    def __init__(self,card = None,teample= None,rank= None,uid= None) -> None:
        self.card = card
        self.teample = teample
        self.rank = rank
        self.uid = uid
        
    async def update(self):
        async with aiohttp.ClientSession() as session:
            async with session.get(api_update.format(uid = self.uid)) as response:
                return await response.json()
    
    async def get_info_character(self, id):
        url = f'https://akasha.cv/api/leaderboards/qusoleum/{id}?type=current'
        
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                if response.status == 200:
                    data = await response.json()
                    return data["data"].get("calculations",{})
                else:
                    return {}
            
    async def get_rank_akasha(self):
        akaska_info = []
        if not self.uid in data_akasha:
            async with aiohttp.ClientSession() as session:
                async with session.get(api_url.format(uid = self.uid)) as response:
                    data = await response.json()
                    for key in data.get("data", []):
                        calculator = key.get("calculations", {}).get("fit", {})
                        if calculator:
                            rank = int(str(calculator.get("ranking", "0")).replace("~", ""))
                            out = int(calculator.get("outOf", "1"))
                            percentage = round((rank / out) * 100)
                            if percentage == 0:
                                percentage = 1
                            akaska_info.append({"id": key["characterId"], "rank": rank, "out": out, "precent": percentage})
                            
                    data_akasha[self.uid] = akaska_info
        return data_akasha[self.uid]
         
    async def creat_logo(self):
        logo = await _of.akasha
        logo = logo.copy()
        
        background = Image.new("RGBA",(264,79) ,(0,0,0,0))
        
        background.alpha_composite(logo)
        
        font = await pill.get_font(12)
        d = ImageDraw.Draw(background)
        d.text((61,55), f'Rank: ~{self.rank.rank}/{self.rank.out}', font = font, fill= (255,255,255,255)) 
        d.text((61,39), f'Top: {self.rank.precent}%', font = font, fill= (255,255,255,255)) 
        
        return background
        
        
    async def start(self):
        if self.rank is None:
            return self.card
        
        logo = await self.creat_logo()

        if self.teample == 1:
            self.card.alpha_composite(logo,(11,409))
        elif self.teample == 2:
            self.card.alpha_composite(logo,(952,13))
        return self.card


# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.
from PIL import Image
from . import pill
import io
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import font_manager

font_path = 'enkacard\\src\\assets\\total\\font\\Genshin_Impact.ttf'

custom_font = font_manager.FontProperties(fname=font_path)
plt.rcParams['font.family'] = custom_font.get_name()

async def create_normalized_radial_chart(data, data_rect, rgb):
    rgb = pill.element_color_text.get(rgb, (149,107,5,255))
    params = [item['name'] for item in data]
    values = [item['value'] for item in data]

    max_values = {item['name']: item['value'] for item in data_rect}

    max_data_rect_values = [item['value'] for item in data_rect]

    num_vars = len(params)
   
    normalized_values = [values[i] / max_values[params[i]] if  max_values[params[i]] != 0 else 0 for i in range(num_vars)]
    normalized_values += [normalized_values[0]]


    angles = np.linspace(0, 2 * np.pi, num_vars, endpoint=False).tolist()
    angles += angles[:1] 

    fig, ax = plt.subplots(subplot_kw=dict(polar=True))
    
    ax.grid(False)

    normalized_values_rect = [max_data_rect_values[i] / max_values[params[i]] if max_values[params[i]] != 0 else 0 for i in range(num_vars)]
    normalized_values_rect += [normalized_values_rect[0]]
    
    num_background_lines = 6

    max_data_rect_value = max(normalized_values_rect)


    step = max_data_rect_value / num_background_lines

    for i in range(1, num_background_lines + 2):
        radii = [step * i] * len(angles)
        radii.append(radii[0]) 
        ax.plot(np.append(angles, angles[0]), radii, color='white', linestyle='-', linewidth=0.5, zorder=0, alpha=0.3)
    
    for i in range(1, len(data_rect)+1):
        radii = [step * i for step in range(len(data_rect))]
        ax.plot([angles[i], angles[i]], [0.16, 1.16], color='white', linestyle='-', linewidth=0.5, zorder=1, alpha=0.3)

    valuess = [item['value'] for item in data_rect]
    valuess += [valuess[0]]
    ax.plot(angles, normalized_values_rect, linewidth=2, color='white', marker='o', linestyle='solid', zorder=0, alpha=0.5)
    for i, (angle, value) in enumerate(zip(angles, normalized_values_rect)):
        if valuess[i] % 1 == 0:
            val = str(valuess[i])
        else:
            val = '{:.1f}%'.format(valuess[i])
            
        ax.annotate(val, (angle, value), textcoords="offset points", xytext=(-10, -10), color = "white", fontsize=7, ha='left', alpha = 1)

    normalized_values_v = []
    for key in normalized_values:
        if key > 1.0:
            key = 1.2
        normalized_values_v.append(key)
        
    ax.plot(angles, normalized_values_v, linewidth=1, color = (rgb[0]/255,rgb[1]/255,rgb[2]/255), marker='o', zorder=1)
    ax.fill(angles, normalized_values_v, color= (rgb[0]/255,rgb[1]/255,rgb[2]/255), alpha=0.3)

    ax.xaxis.grid(linewidth=0) 
    ax.set_yticklabels([])
    ax.set_xticks(angles)
    ax.set_xticklabels(params + params[:1], color='white', fontproperties=custom_font,fontsize=10)
    ax.spines['polar'].set_visible(False)
            
    buffer = io.BytesIO()
    plt.savefig(buffer, format='png', transparent=True)
    buffer.seek(0)

    img = Image.open(buffer)

    plt.close(fig)
    
    return img


# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.
from PIL import Image
import threading
from pathlib import Path
import aiohttp
from io import BytesIO
from cachetools import TTLCache
lock = threading.Lock()

_cache = TTLCache(maxsize=1000, ttl=300)

_BASE_URL = 'https://raw.githubusercontent.com/DEViantUA/EnkaCardData/main/assets/'

assets = Path(__file__).parent.parent / 'assets'

font = str(assets / 'font' / 'Genshin_Impact.ttf')
fontKH = str(assets / 'font' / 'GSEnochian.ttf')

async def change_Font(x):
    global font
    if x == 0:
        font = str(assets / 'font' / 'Genshin_Impact.ttf')
    else:
        font = str(assets / 'font' / 'GSEnochian.ttf')
mapping = {
    "artifact_bg": "one/artifact_bg.png",
    "artifact_bg_none": "one/artifact_bg_none.png",
    "shadow_one": "one/shadow.png",
    "mask_artifact": "one/mask_artifact.png",
    "artifact_up": "one/artifact_up.png",
    "maska_art": "one/maska_art.png",
    "overlay": "one/overlay.png",
    "overlay_dark": "one/overlay_dark.png",
    "sets": "one/sets.png",
    "stats": "one/stats.png",
    "tcv": "one/tcv.png",
    "weapon": "one/weapon.png",
    "closed_const": "total/constant/closed/CLOSED.png",
    "icon_stats": "stats_icon/{prop_id}.png",
    "icon_const_unlock": "total/constant/open/{element}.png",
    "icon_const_lock": "total/constant/closed/{element}.png",
    
    "star1": "total/stars/Star1.png",
    "star2": "total/stars/Star2.png",
    "star3": "total/stars/Star3.png",
    "star4": "total/stars/Star4.png",
    "star5": "total/stars/Star5.png",
    
    "l_star5": "total/stars/5_stars_light.png",
    "l_star4": "total/stars/4_stars_light.png",
    "l_star3": "total/stars/3_stars_light.png",
    "l_star2": "total/stars/2_stars_light.png",
    "l_star1": "total/stars/1_stars_light.png",
    "logo": "total/logo.png",
    "akasha": "total/akasha_logo.png",
    
    "skill_bg": "total/talants/bg.png",
    "skill_count": "total/talants/CoontLow.png",
    
    
    "ab_ac": "profile/ab_ac.png",
    "art_profile_mask": "profile/art_profile_mask.png",
    "avatar_mask": "profile/avatar_mask.png",
    "overlay_profile": "profile/overlay.png",
    "profile_bg_mask": "profile/profile_bg_mask.png",
    "shadow_art_profile": "profile/shadow_art_profile.png",
    
    
    "artifact_set_count": "two/artifact_set_count.png",
    "maska_bg": "two/maska_bg.png",
    "overlay_bg": "two/overlay_l_bg.png",
    "total_mask_bg": "two/total_mask_bg.png",
    "artifact_mask": "two/artifact_mask.png",
    "line_bg": "two/line_bg.png",
    "artifact_eff": "two/artifact_eff.png",
    
    "snow_texture": "two/snow_texture.png",
    "snow": "two/snow.png",
    
    
    "avatar_maska": "profile_two/avatar_maska.png",
    "bg_1": "profile_two/bg_1.png",
    "bg_2": "profile_two/bg_2.png",
    "bg_3": "profile_two/bg_3.png",
    "desc_frame": "profile_two/desc_frame.png",
    "frame_profile": "profile_two/frame_profile.png",
    "icons": "profile_two/icons.png",
    "maska_character": "profile_two/maska_character.png",
    "maska_prof_bg": "profile_two/maska_prof_bg.png"
}

class ImageCache:
    @classmethod
    async def download_image(cls, url):
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                try:
                    image = await response.read()
                finally:
                    await session.close()
                    
        return BytesIO(image)

    @classmethod
    async def _load_image(cls, name):
        url = _BASE_URL + name
        if url in _cache:
            return _cache[url]
        else:
            image_data = await cls.download_image(url)
            image = Image.open(image_data)
            _cache[url] = image
        return image

    async def __getattr__(self, name):
        if name in mapping:
            return await self._load_image(mapping[name])
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")
        
    async def download_icon_stats(self, prop_id):
        if 'icon_stats' in mapping:
            url = mapping['icon_stats'].format(prop_id=prop_id)
            full_url = _BASE_URL + url
            if full_url in _cache:
                return _cache[full_url].copy()
            else:
                image_data = await self.download_image(full_url)
                image = Image.open(image_data)
                _cache[full_url] = image
                return image.copy()
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute 'icon_stats'")

    async def download_icon_constant(self, element, unlock, resizes = None):
        if 'icon_const_unlock' in mapping and "icon_const_lock" in mapping:
            if unlock:
                url = mapping['icon_const_unlock'].format(element=element.upper())
            else:
                url = mapping['icon_const_lock'].format(element=element.upper())
            full_url = _BASE_URL + url
            key = (full_url, resizes, unlock)
            if key in _cache:
                return _cache[key].copy()
            else:
                image_data = await self.download_image(full_url)
                image = Image.open(image_data)
                if not resizes is None:
                    image = image.resize(resizes)
                    
                _cache[full_url] = image
                return image.copy()
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute 'icon_stats'")


from ...utils import pill, git

_of = git.ImageCache()


class Constant:
    def __init__(self,constant, element) -> None:
        self.constant = constant
        self.element = element
    
    async def creat_open(self):
        background = await _of.download_icon_constant(element = f"OPEN_CONST_{self.element}",unlock=self.constant.unlocked, resizes = (108,112))
        icon = await pill.get_dowload_img(self.constant.icon.url, size=((58,58)))
        background.alpha_composite(icon, (25,29))
        
        return background
    
    async def creat_closed(self):
        background = await _of.download_icon_constant(element = f"CLOSE_CONST_{self.element}",unlock=self.constant.unlocked, resizes = (108,112))
        icon = await pill.get_dowload_img(self.constant.icon.url, size=((58,58)))
        background.alpha_composite(icon, (25,29))
        closed_icon = await _of.closed_const
        background.alpha_composite(closed_icon.resize((69,69)), (20,23))
        
        return background
    
    async def start(self):
        if self.constant.unlocked:
            icon = await self.creat_open()
        else:
            icon = await self.creat_closed()
        
        return icon


from PIL import Image,ImageChops
from ...utils import pill, git
import asyncio


_of = git.ImageCache()

class Background:
    def __init__(self,img,element) -> None:
        self.img = img
        self.element = element
        
    async def get_element_color(self):
        if self.element is None:
            #self.element_color = await pill.get_color_art(self.img)
            self.element_color = await  pill.get_background_colors(self.img, 15, common=True, radius=5, quality=800)
            
        else:
            self.element_color = pill.element_color.get(self.element, (149,107,5,255))
            
            
    async def add_overlay(self):
        if self.element is None:
            ll = await pill.light_level(self.element_color)
            if ll < 0.30:
                overlay = await _of.overlay#overlay_dark
                self.background = ImageChops.soft_light(self.background, overlay.convert("RGBA") ) #screen
            else:
                overlay = await _of.overlay_dark
                self.background = ImageChops.soft_light(self.background, overlay.convert("RGBA") )
        else:
            overlay = await _of.overlay
            self.background = ImageChops.soft_light(self.background, overlay.convert("RGBA") )
            
    async def add_mask(self):
        background = Image.new("RGBA", (1511, 1301), (0,0,0,0))
        background_two = Image.new("RGBA", (1511, 1301), (0,0,0,0))
        background.alpha_composite(self.img,(0,308))
        background_two.paste(background,(0,0),self.mask.convert("L"))
        self.background.alpha_composite(background_two)
    
    async def creat_art(self):
        self.img = await pill.get_centr_honkai_art((965,993),self.img)
    
    async def add_shadow(self):
        shadow = await _of.shadow_one
        self.background.alpha_composite(shadow)
        
    async def start(self):
        self.mask =  await _of.maska_art 
        self.mask = self.mask.convert("L")
        self.img = await pill.get_dowload_img(self.img)
        await self.get_element_color()
        self.background = Image.new("RGBA", (1511, 1301), self.element_color)
        await asyncio.gather(self.add_overlay(),self.creat_art())
        await self.add_mask()
        await self.add_shadow()
        
        return self.background 


from PIL import Image,ImageDraw
from ...utils import pill, git, options
import asyncio
import json
import math
_of = git.ImageCache()


class Stat:
    def __init__(self,stats,props,element) -> None:
        self.stats = stats
        self.props = props
        self.element_color = pill.element_color_text.get(element, (149,107,5,255))
        self.line = Image.new("RGBA", (26, 1), (255,255,255,255))
        
    async def creat_prop_icon(self,prop_id):
        background = Image.new("RGBA", (100, 26), (0,0,0,0))

        d = ImageDraw.Draw(background)
        
        icon = await pill.get_icon_add(prop_id, size=(21,20))
        
        main = str(self.new_items[prop_id]["main"])
        sub = str(float('{:.1f}'.format(self.new_items[prop_id]["sub"])))
        if self.new_items[prop_id]["value"] % 1 == 0:
            value = str(self.new_items[prop_id]["value"])
        else:
            value = str('{:.1f}%'.format(self.new_items[prop_id]["value"]))
        x = self.font_13.getlength(main)
        d.text((int(14-x/2),12), main, font= self.font_13, fill=(255,255,255,255))
        
        x = self.font_13.getlength(sub)
        d.text((int(14-x/2),-2), sub, font= self.font_13, fill=(255,255,255,255))
        
        d.text((55,4), value, font= self.font_13, fill=(255,255,255,255))
        
        background.alpha_composite(self.line,(0,13))
        background.alpha_composite(icon,(31,2))
        x = self.font_13.getlength(value)
        background = background.crop((0, 0, 55 + x, 26))
                
        return background
            
            
    async def sort_prop_id(self):
        self.new_items = {}
        for item in self.props:
            for prop_key in item:
                if not prop_key in self.new_items :
                    self.new_items [prop_key] = {"main": item[prop_key]["main"], "sub": item[prop_key]["sub"], "value": item[prop_key]["value"]}
                else:
                    self.new_items [prop_key]['main'] += item[prop_key]['main']
                    self.new_items [prop_key]['sub'] += item[prop_key]['sub']
                    self.new_items [prop_key]['value'] += item[prop_key]['value']
                    
        task = []
        for key in self.new_items:
            task.append(self.creat_prop_icon(key))
        result = await asyncio.gather(*task)
        x = 16
        for prop_icon in result[:9]:
            self.background.alpha_composite(prop_icon, (x,12))
            x += prop_icon.size[0]+10

    async def creat_line_stats(self,stats):
        background = Image.new("RGBA", (411, 50), (0,0,0,0))
        dop_value = 0
        name = options.assets.get_hash_map(stats[0])
        icon,name = await asyncio.gather(pill.get_icon_add(stats[0].replace("_CUR","").replace("_MAX", ""), size=(37,37)), pill.create_image_with_text(name, 21, max_width=215,  color=(255, 255, 255, 255)))
        try:
            value = stats[1].to_percentage_symbol()
        except:
            value = stats[1].to_rounded()
        if stats[0] in self.dop_value:
            dop_value = self.dop_value.get(stats[0], 0)
            up_valur = str(int(value - dop_value["value"]))
            
        
        background.alpha_composite(icon,(0,6))
        background.alpha_composite(name,(49,int(27-name.size[1]/2)))
        
        d = ImageDraw.Draw(background)
        x = self.font_21.getlength(str(value))
        d.text((int(406-x),8), str(value), font= self.font_21, fill=(255,255,255,255))
        if dop_value != 0:
            x = self.font_13.getlength(f"+{dop_value['value']}")
            d.text((int(406-x),32), f"+{dop_value['value']}", font= self.font_13, fill= self.element_color)
            xx = self.font_13.getlength(up_valur)
            d.text((int(406-x-xx-5),32), up_valur, font= self.font_13, fill=(255,255,255,255))

        return {"icon": background,"id": stats[1].id}
            
    async def get_substats(self):
        self.dop_value = {}
        stats_json = json.loads(self.stats.json())
        for key in stats_json:
            if key in options.dopStatAtribute:
                name_dop_value = options.dopStatAtribute.get(key)
                self.dop_value[key] = {"value": math.ceil(stats_json.get(name_dop_value)["value"])}
    
    
    async def add_stats(self):
        x = 22
        y = 68
        for i, key in enumerate(self.sorted_data):
            self.background.alpha_composite(key["icon"],(x,y))
            y += 60
            if i == 4:
                x = 483
                y = 68
                
    async def creat_stats(self):
        task = []
        elementUp = True        
        for key in self.stats:
            if key[1].id in [40,41,42,43,44,45,46]:
                if elementUp:
                    key = max((x for x in self.stats if 40 <= x[1].id <= 46), key=lambda x: x[1].value)
                    elementUp = False
                else:
                    continue
            if key[1].value != 0 and key[0] in options.IconAddTrue:
                task.append(self.creat_line_stats(key))
            
        result = await asyncio.gather(*task)
        self.sorted_data = sorted(result, key=lambda x: x['id'] not in [2000, 2001, 2002])
        
        
        
    async def start(self):
        self.font_13, self.font_21, _, self.background = await asyncio.gather(pill.get_font(13),pill.get_font(21),self.get_substats(),_of.stats)
        self.background = self.background.copy()
        
        await self.sort_prop_id()
        await self.creat_stats()
        await self.add_stats()
        
        return self.background
    
        


from PIL import ImageFont,Image,ImageDraw,ImageStat,ImageFilter
from io import BytesIO
from . import git
import aiohttp,re, json,random
import numpy as np
import colorsys


from PIL import Image
from more_itertools import chunked

from cachetools import TTLCache
from collections import namedtuple
from math import sqrt


__all__ = (
    'get_average_color', 'get_dominant_colors',
    'get_distance_alpha',
    'get_background_alpha', 'get_foreground_alpha',
    'get_background_colors', 'get_foreground_colors',
)

element_color_text = {
   "Wind":(55,199,184,255),
   "Water":(55,167,199,255),
   "Rock":(199,157,55,255),
   "Ice":(55,199,197,255),
   "Electric":(170,55,199,255),
   "Fire":(202,69,69,255),
   "Grass":(60,159,76,255)
}

element_color = {
    "Wind": (5,149,134,255),
    "Water": (5,117,149,255),
    "Rock": (149,107,5,255),
    "Ice": (5,149,147,255),
    "Electric": (120,5,149,255),
    "Fire": (152,19,19,255),
    "Grass": (10,109,26,255)
}

async def apply_opacity(image, opacity=0.2):
    result_image = image.copy()
    alpha = result_image.split()[3]
    alpha = alpha.point(lambda p: int(p * opacity))
    result_image.putalpha(alpha)

    return result_image

async def recolor_image(image, target_color):
    result = Image.new('RGBA', image.size, target_color)
    
    if image.mode == 'RGBA':
        result.putalpha(image.split()[-1])
    
    return result

async def get_stars(level, light = False):
    if light:
        if level == 5:
            return await git.ImageCache().l_star5
        elif level == 4:
            return await git.ImageCache().l_star4
        elif level == 3:
            return await git.ImageCache().l_star3
        elif level == 2:
            return await git.ImageCache().l_star2
        else:
            return await git.ImageCache().l_star1
    
    if level == 5:
        return await git.ImageCache().star5
    elif level == 4:
        return await git.ImageCache().star4
    elif level == 3:
        return await git.ImageCache().star3
    elif level == 2:
        return await git.ImageCache().star2
    else:
        return await git.ImageCache().star1


async def get_icon_add(prop_id, recolor = (255,255,255), size = None):
    if f"{prop_id}_{recolor}_{size}" in cache:
        return cache[f"{prop_id}_{recolor}_{size}"]
    
    icon = await git.ImageCache().download_icon_stats(prop_id = prop_id)
    if "ADD_HURT" not in prop_id:
        icon = await recolor_image(icon,recolor)
    if size:
        icon.thumbnail(size)
        cache[f"{prop_id}_{recolor}_{size}"] = icon.convert("RGBA").copy()
        return cache[f"{prop_id}_{recolor}_{size}"]
    else:
        cache[f"{prop_id}_{recolor}_{size}"] = icon.convert("RGBA").copy()
        return cache[f"{prop_id}_{recolor}_{size}"]

async def get_font(size):
    return ImageFont.truetype(git.font, size)

cache = TTLCache(maxsize=1000, ttl=300)  

async def get_dowload_img(link,size = None, thumbnail_size = None):
    cache_key = json.dumps((link, size, thumbnail_size), sort_keys=True)  # Преобразовываем в строку
        
    if cache_key in cache:
        return cache[cache_key]
    headers_p = {}
    try:
        if "pximg" in link:
            headers_p = {
                "referer": "https://www.pixiv.net/",
            }
        async with aiohttp.ClientSession(headers=headers_p) as session, session.get(link) as r:
            try:
                image = await r.read()
            finally:
                await session.close()
    except:
        raise
    
    image = Image.open(BytesIO(image)).convert("RGBA")
    if size:
        image = image.resize(size)
        cache[cache_key] = image
        return image
    elif thumbnail_size:
        image.thumbnail(thumbnail_size)
        cache[cache_key] = image
        return image
    else:
        cache[cache_key] = image
        return image


async def get_centr_honkai_art(size, file_name):
    background_image = Image.new('RGBA', size, color=(0, 0, 0, 0))
    foreground_image = file_name.convert("RGBA")

    scale = max(size[0] / foreground_image.size[0], size[1] / foreground_image.size[1])
    foreground_image = foreground_image.resize((int(foreground_image.size[0] * scale), int(foreground_image.size[1] * scale)), resample=Image.BICUBIC)

    background_size = background_image.size
    foreground_size = foreground_image.size

    x = background_size[0] // 2 - foreground_size[0] // 2

    if foreground_size[1] > background_size[1]:
        y_offset = max(int(0.3 * (foreground_size[1] - background_size[1])), int(0.5 * (-foreground_size[1])))
        y = -y_offset
    else:
        y = background_size[1] // 2 - foreground_size[1] // 2

    background_image.alpha_composite(foreground_image, (x, y))

    return background_image

Point = namedtuple('Point', ('coords', 'n', 'ct'))
Cluster = namedtuple('Cluster', ('points', 'center', 'n'))

def get_points(img):
    points = []
    w, h = img.size
    for count, color in img.getcolors(w * h):
        points.append(Point(color, 3, count))
    return points

rtoh = lambda rgb: '#%s' % ''.join(('%02x' % p for p in rgb))

def colorz(img, n=3):
    img = img.copy()
    img.thumbnail((200, 200))
    w, h = img.size

    points = get_points(img)
    clusters = kmeans(points, n, 1)
    rgbs = [map(int, c.center.coords) for c in clusters]
    
    return map(rtoh, rgbs)

def euclidean(p1, p2):
    return sqrt(sum([
        (p1.coords[i] - p2.coords[i]) ** 2 for i in range(p1.n)
    ]))

def calculate_center(points, n):
    vals = [0.0 for i in range(n)]
    plen = 0
    for p in points:
        plen += p.ct
        for i in range(n):
            vals[i] += (p.coords[i] * p.ct)
    return Point([(v / plen) for v in vals], n, 1)

def kmeans(points, k, min_diff):
    clusters = [Cluster([p], p, p.n) for p in random.sample(points, k)]

    while 1:
        plists = [[] for i in range(k)]

        for p in points:
            smallest_distance = float('Inf')
            for i in range(k):
                distance = euclidean(p, clusters[i].center)
                if distance < smallest_distance:
                    smallest_distance = distance
                    idx = i
            plists[idx].append(p)

        diff = 0
        for i in range(k):
            old = clusters[i]
            center = calculate_center(plists[i], old.n)
            new = Cluster(plists[i], center, old.n)
            clusters[i] = new
            diff = max(diff, euclidean(old.center, new.center))

        if diff < min_diff:
            break
    
    return clusters

async def light_level(pixel_color):
    h, l, s = colorsys.rgb_to_hls(*(x / 255 for x in pixel_color[:3]))    
    return l

async def _get_light_pixel_color(pixel_color):
    h, l, s = colorsys.rgb_to_hls(*(x / 255 for x in pixel_color[:3]))
    l = min(max(0.2, l), 0.8)

    return tuple(round(x * 255) for x in colorsys.hls_to_rgb(h, l, s))
  
async def _get_dark_pixel_color(pixel_color):
    h, l, s = colorsys.rgb_to_hls(*(x / 255 for x in pixel_color[:3]))
    l = min(max(0.8, l), 0.2)
    a = tuple(round(x * 255) for x in colorsys.hls_to_rgb(h, l, s))
    
    return  a


async def get_color_art(img):
    a = colorz(img, n = 2)
    a = list(a)[0].lstrip('#')
    color_palette = tuple(int(a[i:i+2], 16) for i in (0, 2, 4))
    
    ll = await light_level(color_palette)
    
    if ll < 45:
        color_palette = await _get_light_pixel_color(color_palette)
    elif ll > 200:
        color_palette = await _get_dark_pixel_color(color_palette)
    return color_palette


async def create_image_with_text(text, font_size, max_width=336, color=(255, 255, 255, 255), alg="Left"):
    
    key = (text, font_size, max_width, color, alg)
    if key in cache:
        return cache[key]
    
    font = await get_font(font_size)

    lines = []
    line = []
    for word in text.split():
        if line:
            temp_line = line + [word]
            temp_text = ' '.join(temp_line)
            temp_width = font.getmask(temp_text).getbbox()[2]
            if temp_width <= max_width:
                line = temp_line
            else:
                lines.append(line)
                line = [word]
        else:
            line = [word]
    if line:
        lines.append(line)

    width,height = 0, 0
    
    for line in lines:
        line_width = font.getmask(' '.join(line)).getbbox()[2]
        width = max(width, line_width)
        height += font.getmask(' '.join(line)).getbbox()[3]

    img = Image.new('RGBA', (min(width, max_width), height + (font_size)), color=(255, 255, 255, 0))

    draw = ImageDraw.Draw(img)
    
    y_text = 0
    for line_num, line in enumerate(lines):
        text_width, text_height = font.getmask(' '.join(line)).getbbox()[2:]
        if alg == "center" and line_num > 0:
            x_text = (max_width - text_width) // 2
        else:
            x_text = 0
        draw.text((x_text, y_text), ' '.join(line), font=font, fill=color)
        y_text += text_height + 5
        
    cache[key] = img
    
    return img


async def get_average_color(image):
    if image.mode != 'RGBA':
        image = image.convert('RGBA')
    
    channels = image.split()
    
    return (
        round(np.average(channels[0], weights=channels[-1])),
        round(np.average(channels[1], weights=channels[-1])),
        round(np.average(channels[2], weights=channels[-1])),
    )


async def get_dominant_colors(
    image,
    number,
    *,
    dither=Image.Quantize.FASTOCTREE,
    common=True,
):
    if image.mode != 'RGB':
        if image.mode != 'RGBA':
            image = image.convert('RGBA')
        
        if not common:
            width = image.width
            height = image.height
            
            image = Image.fromarray(np.array([np.repeat(
                np.reshape(image.convert('RGB'), (width * height, 3)),
                np.reshape(image.split()[-1], width * height),
                0,
            )]), 'RGB')
    
    if image.mode == 'RGBA':
        if dither == Image.Quantize.FASTOCTREE:
            simple_image = image.copy()
            simple_image.putalpha(255)
        else:
            simple_image = image.convert('RGB')
    else:
        simple_image = image
    
    reduced = simple_image.quantize(dither=dither, colors=number)
    
    palette = [*chunked(reduced.getpalette(), 3)]
    
    if common and image.mode == 'RGBA':
        alpha = np.array(image.split()[-1])
        
        colors = sorted((
            (
                np.sum(alpha * reduced.point([0] * i + [1] + [0] * (255 - i))),
                tuple(palette[i]),
            )
            for _, i in reduced.getcolors()
        ), reverse=True)
    else:
        colors = [
            (n, tuple(palette[i]))
            for n, i in sorted(reduced.getcolors(), reverse=True)
        ]
    
    return tuple(colors)


async def get_distance_alpha(image, converter=(lambda x: x)):
    width = image.width
    height = image.height
    
    radius = np.hypot(1, 1)
    
    return Image.fromarray(np.fromfunction(
        lambda y, x: np.uint8(255 * converter(np.hypot(
            2 * x / (width - 1) - 1,
            2 * y / (height - 1) - 1,
        ) / radius)),
        (height, width),
    ), 'L')


async def get_background_alpha(image):
    return await get_distance_alpha(
        image,
        lambda x: x * np.sin(x * np.pi / 2),
    )


async def get_foreground_alpha(image):
    return await get_distance_alpha(
        image,
        lambda x: 1 - x * np.sin(x * np.pi / 2),
    )

async def get_background_colors(image,number,*,common=False,radius=1,quality=None):
    if quality is not None:
        image = image.copy()
        image.thumbnail((quality, quality), 0)
    
    if radius > 1:
        image = image.filter(ImageFilter.BoxBlur(radius))
    
    filtered_image = image.convert('RGB')
    
    if image.mode != 'RGBA':
        filtered_image.putalpha(await get_background_alpha(image))
    else:
        filtered_image.putalpha(Image.fromarray(np.uint8(
            np.uint16(await get_background_alpha(image))
            * image.split()[-1]
            / 255
        ), 'L'))
    
    color_palette = await get_dominant_colors(filtered_image, number, common=common)
    color_palette = color_palette[0][1]
    ll = await light_level(color_palette)
    if ll < 0.15:
        color_palette = await _get_light_pixel_color(color_palette)
    elif ll > 0.80:
        color_palette = await _get_dark_pixel_color(color_palette)
        
        
    return color_palette
     


async def get_foreground_colors(image,number,*,common=False,radius=1,quality=None):
    if quality is not None:
        image = image.copy()
        image.thumbnail((quality, quality), 0)
    
    if radius > 1:
        image = image.filter(ImageFilter.BoxBlur(radius))
    
    filtered_image = image.convert('RGB')
    
    if image.mode != 'RGBA':
        filtered_image.putalpha(await get_foreground_alpha(image))
    else:
        filtered_image.putalpha(Image.fromarray(np.uint8(
            np.uint16(await  get_foreground_alpha(image))
            * image.split()[-1]
            / 255
        ), 'L'))
    
    return await  get_dominant_colors(filtered_image, number, common=common)


async def apply_opacity(image, opacity=0.2):
    result_image = image.copy()
    alpha = result_image.split()[3]
    alpha = alpha.point(lambda p: int(p * opacity))
    result_image.putalpha(alpha)

    return result_image
    
class GradientGenerator:
    def __init__(self, source_img_path):
        self.source_img = source_img_path
        self.frame = ()
        self.source_width, self.source_height = self.source_img.size

    async def generate(self, width, height, left = False):
        gradient_img = Image.new("RGB", (width, height))

        # Вычисляем ширину и высоту каждой полосы градиента
        top_height = height // 3
        bottom_height = height // 3
        center_height = height - top_height - bottom_height
        # Определяем координаты точек, с которых будем брать цвета
        if left:
            left = 3
            right = 4
        else:
            left = self.source_width - 142
            right = self.source_width - 141
        top_1 = 1
        bottom_1 = top_height - 1
        top_2 = top_height + 1
        bottom_2 = top_height + center_height - 1
        top_3 = top_height + center_height + 1
        bottom_3 = height - 2

        top_color = await self._get_pixel_color(left, top_1, right, bottom_1)
        ll = await light_level(top_color)
        if ll < 45:
            top_color = await _get_light_pixel_color(top_color)
        elif ll > 200:
            top_color = await _get_dark_pixel_color(top_color)

        center_color = await self._get_pixel_color(left, top_2, right, bottom_2)
        
        ll = await light_level(center_color)
        if ll < 45:
            center_color = await _get_light_pixel_color(center_color)
        elif ll > 200:
            center_color = await _get_dark_pixel_color(center_color)

        bottom_color = await self._get_pixel_color(left, top_3, right, bottom_3)
        ll = await light_level(bottom_color)
        if ll < 45:
            bottom_color = await _get_light_pixel_color(bottom_color)
        elif ll > 200:
            bottom_color = await _get_dark_pixel_color(bottom_color)

        for y in range(top_height):
            for x in range(width):
                ratio = y / (top_height - 1)
                gradient_color = self._get_interpolated_color(top_color, center_color, ratio)
                gradient_img.putpixel((x, y), gradient_color)

        for y in range(center_height):
            for x in range(width):
                ratio = y / (center_height - 1)
                gradient_color = self._get_interpolated_color(center_color, bottom_color, ratio)
                gradient_img.putpixel((x, y + top_height), gradient_color)

        for y in range(bottom_height):
            for x in range(width):
                gradient_color = bottom_color
                gradient_img.putpixel((x, y + top_height + center_height), gradient_color)

        return gradient_img

    async def _get_pixel_color(self, left, top, right, bottom):
        cropped_img = self.source_img.crop((left, top, right, bottom))
        resized_img = cropped_img.convert("RGB").resize((1, 1))
        pixel_color = resized_img.getpixel((0, 0))
        
        return pixel_color
    
    def _get_interpolated_color(self, start_color, end_color, ratio):
        return tuple(int(start_color[i] + (end_color[i] - start_color[i]) * ratio) for i in range(3))


from PIL import Image,ImageDraw
from ...utils import pill, git

_of = git.ImageCache()

class Skill:
    def __init__(self, skill,element) -> None:
        self.skill = skill
        self.element_color = pill.element_color_text.get(element, (149,107,5,255))
    
    async def add_icon(self):
        icon = await pill.get_dowload_img(self.skill.icon.url, size= (63,62))
        self.background.alpha_composite(icon,(24,5))
    
    async def add_count(self):
        if self.skill.is_boosted:
            self.count = await pill.recolor_image(self.count,self.element_color[:3])
        
        font_25 = await pill.get_font(25)
        
        d = ImageDraw.Draw(self.count)
        
        x = font_25.getlength(str(self.skill.level))
        d.text((int(22-x/2),6), str(self.skill.level), font= font_25, fill=(255,255,255,255))
    
    async def start(self):
        self.background = await _of.skill_bg
        self.background = self.background.copy()
        self.count = await _of.skill_count
        self.count = self.count.copy()
        
        await self.add_icon()
        await self.add_count()
        
        background = Image.new("RGBA", (82, 81), (0,0,0,0))
        background.alpha_composite(self.background.resize((82,64)))
        background.alpha_composite(self.count,(20,42))

        return background


from ...utils import options

class Prop:
    def __init__(self, artifact) -> None:
        self.artifact = artifact
        self.prop_value = {}
        self.props = {}
        
    async def creat_prop_value(self):
        for key in self.artifact.detail.substats:      
            self.prop_value[key.prop_id] = key.value
    
    async def creat_prop(self):
        for key in self.artifact.props:
            value = self.prop_value.get(key.prop_id, 0)
            self.props[key.prop_id] = {"name": key.prop_id, "main": options.data_prop_json.get(str(key.id), {}).get("position", 0), "sub": options.data_prop_json.get(str(key.id), {}).get("efficiency", 0), "value": value}
    
    async def start(self):
        await self.creat_prop_value()
        await self.creat_prop()
        
        return self.props
        
        
    
    


from pydantic import BaseModel
from typing import Optional

class ImageInfo(BaseModel):
    filename: Optional[str]
    name: Optional[str]
    mime: Optional[str]
    extension: Optional[str]
    url: Optional[str]

class DataInfo(BaseModel):
    id: Optional[str]
    title: Optional[str]
    url_viewer: Optional[str]
    url: Optional[str]
    display_url: Optional[str]
    width: Optional[int]
    height: Optional[int]
    size: Optional[int]
    time: Optional[int]
    expiration: Optional[int]
    image: Optional[ImageInfo]
    thumb: Optional[ImageInfo]
    medium: Optional[ImageInfo]
    delete_url: Optional[str]

class EnkaCardLink(BaseModel):
    data: Optional[DataInfo]
    success: Optional[bool]
    status: Optional[int]


FOLDER="ExcelBinOutput"
LANG_FOLDER="TextMap"

# Avatars Data
AVATAR="AvatarExcelConfigData.json" #DONE

# Costume Data
COSTUME="AvatarCostumeExcelConfigData.json" #DONE

# Avatar Skills Data
SKILLDEPOT="AvatarSkillDepotExcelConfigData.json" # Skill Depot #DONE
SKILLS="AvatarSkillExcelConfigData.json" # Skills #DONE
TALENTS="AvatarTalentExcelConfigData.json" # AKA. Constellations #DONE

# Equipment Data
ARTIFACTS="ReliquaryExcelConfigData.json" #DONE
WEAPONS="WeaponExcelConfigData.json" #DONE
ARTIFACTS_SETS="EquipAffixExcelConfigData.json" #DONE

# Fight Props
FIGHT_PROPS="ManualTextMapConfigData.json" #DONE

# Props Map
PROPS_MAP="ProudSkillExcelConfigData.json" #NONE

# Namecards
NAMECARDS="MaterialExcelConfigData.json" #DONE

# Artifact main & sub
ARTIFACT_PROPS_MAIN="ReliquaryMainPropExcelConfigData.json"
ARTIFACT_PROPS_SUB="ReliquaryAffixExcelConfigData.json"


ENVKEY = [
    "AVATAR",
    "SKILLDEPOT",
    "SKILLS",
    "TALENTS",
    "ARTIFACTS",
    "WEAPONS",
    "FIGHT_PROPS",
    "NAMECARDS",
    "ARTIFACTS_SETS",
    "COSTUME",
    "PROPS_MAP",
    "ARTIFACT_PROPS_MAIN",
    "ARTIFACT_PROPS_SUB"
]

LANGKEY = [
    'CHS',
    'CHT',
    'DE',
    'EN',
    'ES',
    'FR',
    'ID',
    'IT',
    'JP',
    'KR',
    'PT',
    'RU',
    'TH',
    'TR',
    'VI'
]

LANGKEY_FILE = [
    'artifact_sets',
    'artifacts',
    'characters',
    'constellations',
    'namecards',
    'skills',
    'weapons'
]


data_prop_json = {
	"101021": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.7998661408903833,
		"position": 1
	},
	"101022": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 1,
		"position": 2
	},
	"101031": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.8013698223808419,
		"position": 1
	},
	"101032": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 1,
		"position": 2
	},
	"101051": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.799999951093626,
		"position": 1
	},
	"101052": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 1,
		"position": 2
	},
	"101061": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.8013698223808419,
		"position": 1
	},
	"101062": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 1,
		"position": 2
	},
	"101081": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.800865831025008,
		"position": 1
	},
	"101082": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 1,
		"position": 2
	},
	"101091": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.8021977881396791,
		"position": 1
	},
	"101092": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 1,
		"position": 2
	},
	"101201": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.8041236658084417,
		"position": 1
	},
	"101202": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 1,
		"position": 2
	},
	"101221": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.7989690355415727,
		"position": 1
	},
	"101222": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 1,
		"position": 2
	},
	"101231": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.8024691315440212,
		"position": 1
	},
	"101232": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 1,
		"position": 2
	},
	"101241": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.7993138779408093,
		"position": 1
	},
	"101242": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 1,
		"position": 2
	},
	"201021": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.700000010640718,
		"position": 1
	},
	"201022": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.8500697818291011,
		"position": 2
	},
	"201023": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 1,
		"position": 3
	},
	"201031": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.6995708401537059,
		"position": 1
	},
	"201032": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.8497854200768529,
		"position": 2
	},
	"201033": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 1,
		"position": 3
	},
	"201051": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.700214117238633,
		"position": 1
	},
	"201052": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.8501070586193165,
		"position": 2
	},
	"201053": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 1,
		"position": 3
	},
	"201061": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.6995708401537059,
		"position": 1
	},
	"201062": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.8497854200768529,
		"position": 2
	},
	"201063": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 1,
		"position": 3
	},
	"201081": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.6996403138377545,
		"position": 1
	},
	"201082": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.8489208343106637,
		"position": 2
	},
	"201083": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 1,
		"position": 3
	},
	"201091": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.7010309324542161,
		"position": 1
	},
	"201092": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.8522336763160644,
		"position": 2
	},
	"201093": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 1,
		"position": 3
	},
	"201201": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.7032258250586824,
		"position": 1
	},
	"201202": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.8516129125293411,
		"position": 2
	},
	"201203": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 1,
		"position": 3
	},
	"201221": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.7009646525642665,
		"position": 1
	},
	"201222": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.8488746119364413,
		"position": 2
	},
	"201223": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 1,
		"position": 3
	},
	"201231": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.6988417152242902,
		"position": 1
	},
	"201232": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.8494208216537456,
		"position": 2
	},
	"201233": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 1,
		"position": 3
	},
	"201241": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.6998928470745912,
		"position": 1
	},
	"201242": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.8499463979833213,
		"position": 2
	},
	"201243": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 1,
		"position": 3
	},
	"301021": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.700000010640718,
		"position": 1
	},
	"301022": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.8000000425628723,
		"position": 2
	},
	"301023": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.9000000212814362,
		"position": 3
	},
	"301024": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 1,
		"position": 4
	},
	"301031": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.699999978712627,
		"position": 1
	},
	"301032": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.800000021287373,
		"position": 2
	},
	"301033": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.9000000106436865,
		"position": 3
	},
	"301034": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"301051": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.700214117238633,
		"position": 1
	},
	"301052": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.7997858317081389,
		"position": 2
	},
	"301053": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.8993575461776449,
		"position": 3
	},
	"301054": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 1,
		"position": 4
	},
	"301061": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.699999978712627,
		"position": 1
	},
	"301062": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.800000021287373,
		"position": 2
	},
	"301063": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.9000000106436865,
		"position": 3
	},
	"301064": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"301081": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.7002700675271825,
		"position": 1
	},
	"301082": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.8001800736312134,
		"position": 2
	},
	"301083": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.9000900368156067,
		"position": 3
	},
	"301084": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 1,
		"position": 4
	},
	"301091": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.7002288624079749,
		"position": 1
	},
	"301092": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.8009153643849788,
		"position": 2
	},
	"301093": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.8993134980229961,
		"position": 3
	},
	"301094": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"301201": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.6995708401537059,
		"position": 1
	},
	"301202": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.7982832806729697,
		"position": 2
	},
	"301203": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.9012875594807361,
		"position": 3
	},
	"301204": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 1,
		"position": 4
	},
	"301221": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.6995708401537059,
		"position": 1
	},
	"301222": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.8004292397881477,
		"position": 2
	},
	"301223": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.9012875594807361,
		"position": 3
	},
	"301224": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 1,
		"position": 4
	},
	"301231": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.6992288170077031,
		"position": 1
	},
	"301232": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.7994858620441655,
		"position": 2
	},
	"301233": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.8997429549635376,
		"position": 3
	},
	"301234": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 1,
		"position": 4
	},
	"301241": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.6997855698371146,
		"position": 1
	},
	"301242": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.7998570238353141,
		"position": 2
	},
	"301243": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.8999285460018005,
		"position": 3
	},
	"301244": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 1,
		"position": 4
	},
	"401021": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.7000000127688611,
		"position": 1
	},
	"401022": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.7999999872311389,
		"position": 2
	},
	"401023": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.9000000255377223,
		"position": 3
	},
	"401024": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 1,
		"position": 4
	},
	"401031": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.6995708401537059,
		"position": 1
	},
	"401032": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.8004292397881477,
		"position": 2
	},
	"401033": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.9012875594807361,
		"position": 3
	},
	"401034": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"401051": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.6998714684861824,
		"position": 1
	},
	"401052": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.8001285008687569,
		"position": 2
	},
	"401053": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.899742906327304,
		"position": 3
	},
	"401054": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 1,
		"position": 4
	},
	"401061": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.6995708401537059,
		"position": 1
	},
	"401062": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.8004292397881477,
		"position": 2
	},
	"401063": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.9012875594807361,
		"position": 3
	},
	"401064": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"401081": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.6997840020416457,
		"position": 1
	},
	"401082": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.800215946464062,
		"position": 2
	},
	"401083": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.9001079732320311,
		"position": 3
	},
	"401084": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 1,
		"position": 4
	},
	"401091": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.6998285024582124,
		"position": 1
	},
	"401092": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.799313882035583,
		"position": 2
	},
	"401093": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.9005145565239961,
		"position": 3
	},
	"401094": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"401201": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.7009646525642665,
		"position": 1
	},
	"401202": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.8006431216735538,
		"position": 2
	},
	"401203": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.900321590782841,
		"position": 3
	},
	"401204": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 1,
		"position": 4
	},
	"401221": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.6993569082725104,
		"position": 1
	},
	"401222": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.7990353773817976,
		"position": 2
	},
	"401223": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.900321590782841,
		"position": 3
	},
	"401224": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 1,
		"position": 4
	},
	"401231": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.70077217786434,
		"position": 1
	},
	"401232": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.7992277861772604,
		"position": 2
	},
	"401233": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.8996138571302306,
		"position": 3
	},
	"401234": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 1,
		"position": 4
	},
	"401241": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.7002681333376832,
		"position": 1
	},
	"401242": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.8000000204541412,
		"position": 2
	},
	"401243": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.9002681640188951,
		"position": 3
	},
	"401244": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 1,
		"position": 4
	},
	"501021": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.7000167527458159,
		"position": 1
	},
	"501022": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.8,
		"position": 2
	},
	"501023": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 0.9000167527458159,
		"position": 3
	},
	"501024": {
		"propType": "FIGHT_PROP_HP",
		"efficiency": 1,
		"position": 4
	},
	"501031": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.6998285024582124,
		"position": 1
	},
	"501032": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.799313882035583,
		"position": 2
	},
	"501033": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 0.9005145565239961,
		"position": 3
	},
	"501034": {
		"propType": "FIGHT_PROP_HP_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"501051": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.700257036056831,
		"position": 1
	},
	"501052": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.7999999901935807,
		"position": 2
	},
	"501053": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 0.9002570458632503,
		"position": 3
	},
	"501054": {
		"propType": "FIGHT_PROP_ATTACK",
		"efficiency": 1,
		"position": 4
	},
	"501061": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.6998285024582124,
		"position": 1
	},
	"501062": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.799313882035583,
		"position": 2
	},
	"501063": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 0.9005145565239961,
		"position": 3
	},
	"501064": {
		"propType": "FIGHT_PROP_ATTACK_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"501081": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.6997840617661291,
		"position": 1
	},
	"501082": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.8000000329563484,
		"position": 2
	},
	"501083": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 0.8997840288097807,
		"position": 3
	},
	"501084": {
		"propType": "FIGHT_PROP_DEFENSE",
		"efficiency": 1,
		"position": 4
	},
	"501091": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.6995884908250543,
		"position": 1
	},
	"501092": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.7997256775838287,
		"position": 2
	},
	"501093": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 0.8998628643426032,
		"position": 3
	},
	"501094": {
		"propType": "FIGHT_PROP_DEFENSE_PERCENT",
		"efficiency": 1,
		"position": 4
	},
	"501201": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.6992288170077031,
		"position": 1
	},
	"501202": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.7994858620441655,
		"position": 2
	},
	"501203": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 0.8997429549635376,
		"position": 3
	},
	"501204": {
		"propType": "FIGHT_PROP_CRITICAL",
		"efficiency": 1,
		"position": 4
	},
	"501221": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.7001287406686748,
		"position": 1
	},
	"501222": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.8005148188410904,
		"position": 2
	},
	"501223": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 0.8996139218275844,
		"position": 3
	},
	"501224": {
		"propType": "FIGHT_PROP_CRITICAL_HURT",
		"efficiency": 1,
		"position": 4
	},
	"501231": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.6990740469264689,
		"position": 1
	},
	"501232": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.7993827171139947,
		"position": 2
	},
	"501233": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 0.8996913298124741,
		"position": 3
	},
	"501234": {
		"propType": "FIGHT_PROP_CHARGE_EFFICIENCY",
		"efficiency": 1,
		"position": 4
	},
	"501241": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.7001287030773611,
		"position": 1
	},
	"501242": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.800085802051574,
		"position": 2
	},
	"501243": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 0.900042901025787,
		"position": 3
	},
	"501244": {
		"propType": "FIGHT_PROP_ELEMENT_MASTERY",
		"efficiency": 1,
		"position": 4
	}
}